@@ -333,13 +333,25 @@ func (r *ReconcileSriovNetworkNodePolicy) syncSriovNetworkNodeState(np *sriovnet
 	} else {
 		logger.Info("SriovNetworkNodeState already exists, updating")
 		found.Spec = ns.Spec
+
+		// Previous Policy Priority(ppp) records the priority of previous evaluated policy in node policy list.
+		// Since node policy list is already sorted with priority number, comparing current priority with ppp shall
+		// be sufficient.
+		// ppp is set to 100 as initial value to avoid matching with the first policy in policy list, although
+		// it should not matter since the flag used in p.Apply() will only be applied when VF partition is detected.
+		ppp := 100
 		for _, p := range npl.Items {
 			if p.Name == "default" {
 				continue
 			}
 			if p.Selected(node) {
 				fmt.Printf("apply policy %s for node %s\n", p.Name, node.Name)
-				p.Apply(found)
+				// Merging only for policies with the same priority (ppp == p.Spec.Priority)
+				// This boolean flag controls merging of PF configuration (e.g. mtu, numvfs etc)
+				// when VF partition is configured.
+				p.Apply(found, bool(ppp == p.Spec.Priority))
+				// record the evaluated policy priority for next loop
+				ppp = p.Spec.Priority
 			}
 		}
 		found.Spec.DpConfigVersion = cksum
@@ -364,6 +376,7 @@ func (r *ReconcileSriovNetworkNodePolicy) syncPluginDaemonObjs(dp *sriovnetworkv
 	data := render.MakeRenderData()
 	data.Data["Namespace"] = Namespace
 	data.Data["SRIOVCNIImage"] = os.Getenv("SRIOV_CNI_IMAGE")
+	data.Data["SRIOVInfiniBandCNIImage"] = os.Getenv("SRIOV_INFINIBAND_CNI_IMAGE")
 	data.Data["SRIOVDevicePluginImage"] = os.Getenv("SRIOV_DEVICE_PLUGIN_IMAGE")
 	data.Data["ReleaseVersion"] = os.Getenv("RELEASEVERSION")
 	data.Data["ResourcePrefix"] = os.Getenv("RESOURCE_PREFIX")
@@ -426,6 +439,7 @@ func (r *ReconcileSriovNetworkNodePolicy) tryDeleteDsPods(namespace, name string
 		}
 	} else {
 		ds.Spec.Template.Spec.NodeSelector = map[string]string{"beta.kubernetes.io/os": "none"}
+		ds.Spec.Template.Spec.Affinity = &corev1.Affinity{}
 		err = r.client.Update(context.TODO(), ds)
 		if err != nil {
 			logger.Error(err, "Fail to update DaemonSet", "Namespace", namespace, "Name", name)
@@ -499,7 +513,27 @@ func (r *ReconcileSriovNetworkNodePolicy) syncDaemonSet(cr *sriovnetworkv1.Sriov
 }
 
 func setDsNodeAffinity(pl *sriovnetworkv1.SriovNetworkNodePolicyList, ds *appsv1.DaemonSet) error {
-	terms := nodeSelectorTermsForPolicyList(pl.Items)
+	terms := []corev1.NodeSelectorTerm{}
+	for _, p := range pl.Items {
+		nodeSelector := corev1.NodeSelectorTerm{}
+		if len(p.Spec.NodeSelector) == 0 {
+			continue
+		}
+		for k, v := range p.Spec.NodeSelector {
+			expressions := []corev1.NodeSelectorRequirement{}
+			exp := corev1.NodeSelectorRequirement{
+				Operator: corev1.NodeSelectorOpIn,
+				Key:      k,
+				Values:   []string{v},
+			}
+			expressions = append(expressions, exp)
+			nodeSelector = corev1.NodeSelectorTerm{
+				MatchExpressions: expressions,
+			}
+		}
+		terms = append(terms, nodeSelector)
+	}
+
 	if len(terms) > 0 {
 		ds.Spec.Template.Spec.Affinity = &corev1.Affinity{
 			NodeAffinity: &corev1.NodeAffinity{
@@ -512,36 +546,6 @@ func setDsNodeAffinity(pl *sriovnetworkv1.SriovNetworkNodePolicyList, ds *appsv1
 	return nil
 }
 
-func nodeSelectorTermsForPolicyList(policies []sriovnetworkv1.SriovNetworkNodePolicy) []corev1.NodeSelectorTerm {
-	terms := []corev1.NodeSelectorTerm{}
-	for _, p := range policies {
-		nodeSelector := corev1.NodeSelectorTerm{}
-		if len(p.Spec.NodeSelector) == 0 {
-			continue
-		}
-		expressions := []corev1.NodeSelectorRequirement{}
-		for k, v := range p.Spec.NodeSelector {
-			exp := corev1.NodeSelectorRequirement{
-				Operator: corev1.NodeSelectorOpIn,
-				Key:      k,
-				Values:   []string{v},
-			}
-			expressions = append(expressions, exp)
-		}
-		// sorting is needed to keep the daemon spec stable.
-		// the items are popped in a random order from the map
-		sort.Slice(expressions, func(i, j int) bool {
-			return expressions[i].Key < expressions[j].Key
-		})
-		nodeSelector = corev1.NodeSelectorTerm{
-			MatchExpressions: expressions,
-		}
-		terms = append(terms, nodeSelector)
-	}
-
-	return terms
-}
-
 // renderDsForCR returns a busybox pod with the same name/namespace as the cr
 func renderDsForCR(path string, data *render.RenderData) ([]*uns.Unstructured, error) {
 	logger := log.WithName("renderDsForCR")
@@ -566,65 +570,77 @@ func renderDevicePluginConfigData(pl *sriovnetworkv1.SriovNetworkNodePolicyList)
 		}
 
 		found, i := resourceNameInList(p.Spec.ResourceName, &rcl)
+		netDeviceSelectors := dptypes.NetDeviceSelectors{}
 		if found {
-			if p.Spec.NicSelector.Vendor != "" && !sriovnetworkv1.StringInArray(p.Spec.NicSelector.Vendor, rcl.ResourceList[i].Selectors.Vendors) {
-				rcl.ResourceList[i].Selectors.Vendors = append(rcl.ResourceList[i].Selectors.Vendors, p.Spec.NicSelector.Vendor)
+			if err := json.Unmarshal(*rcl.ResourceList[i].Selectors, &netDeviceSelectors); err != nil {
+				return rcl, err
+			}
+
+			if p.Spec.NicSelector.Vendor != "" && !sriovnetworkv1.StringInArray(p.Spec.NicSelector.Vendor, netDeviceSelectors.Vendors) {
+				netDeviceSelectors.Vendors = append(netDeviceSelectors.Vendors, p.Spec.NicSelector.Vendor)
 			}
 			if p.Spec.NicSelector.DeviceID != "" {
 				var deviceID string
 				if p.Spec.NumVfs == 0 {
 					deviceID = p.Spec.NicSelector.DeviceID
 				} else {
-					deviceID = sriovnetworkv1.SriovPfVfMap[p.Spec.NicSelector.DeviceID]
+					deviceID = sriovnetworkv1.GetVfDeviceId(p.Spec.NicSelector.DeviceID)
 				}
 
-				if !sriovnetworkv1.StringInArray(p.Spec.NicSelector.DeviceID, rcl.ResourceList[i].Selectors.Devices) {
-					rcl.ResourceList[i].Selectors.Devices = append(rcl.ResourceList[i].Selectors.Devices, deviceID)
+				if !sriovnetworkv1.StringInArray(deviceID, netDeviceSelectors.Devices) && deviceID != "" {
+					netDeviceSelectors.Devices = append(netDeviceSelectors.Devices, deviceID)
 				}
 			}
 			if len(p.Spec.NicSelector.PfNames) > 0 {
-				rcl.ResourceList[i].Selectors.PfNames = sriovnetworkv1.UniqueAppend(rcl.ResourceList[i].Selectors.PfNames, p.Spec.NicSelector.PfNames...)
+				netDeviceSelectors.PfNames = sriovnetworkv1.UniqueAppend(netDeviceSelectors.PfNames, p.Spec.NicSelector.PfNames...)
 			}
+			// Removed driver constraint for "netdevice" DeviceType
 			if p.Spec.DeviceType == "vfio-pci" {
-				rcl.ResourceList[i].Selectors.Drivers = sriovnetworkv1.UniqueAppend(rcl.ResourceList[i].Selectors.Drivers, p.Spec.DeviceType)
-			} else {
-				if p.Spec.NumVfs > 0 {
-					///////////////////////////////////
-					// TODO: remove unsupport VF driver
-					///////////////////////////////////
-					rcl.ResourceList[i].Selectors.Drivers = sriovnetworkv1.UniqueAppend(rcl.ResourceList[i].Selectors.Drivers, "iavf", "mlx5_core", "ixgbevf", "i40evf")
-				}
+				netDeviceSelectors.Drivers = sriovnetworkv1.UniqueAppend(netDeviceSelectors.Drivers, p.Spec.DeviceType)
 			}
+
+			netDeviceSelectorsMarshal, err := json.Marshal(netDeviceSelectors)
+			if err != nil {
+				return rcl, err
+			}
+			rawNetDeviceSelectors := json.RawMessage(netDeviceSelectorsMarshal)
+			rcl.ResourceList[i].Selectors = &rawNetDeviceSelectors
 			logger.Info("Update resource", "Resource", rcl.ResourceList[i])
 		} else {
 			rc := &dptypes.ResourceConfig{
 				ResourceName: p.Spec.ResourceName,
-				IsRdma:       p.Spec.IsRdma,
 			}
+			netDeviceSelectors.IsRdma = p.Spec.IsRdma
+
 			if p.Spec.NicSelector.Vendor != "" {
-				rc.Selectors.Vendors = append(rc.Selectors.Vendors, p.Spec.NicSelector.Vendor)
+				netDeviceSelectors.Vendors = append(netDeviceSelectors.Vendors, p.Spec.NicSelector.Vendor)
 			}
 			if p.Spec.NicSelector.DeviceID != "" {
+				var deviceID string
 				if p.Spec.NumVfs == 0 {
-					rc.Selectors.Devices = append(rc.Selectors.Devices, p.Spec.NicSelector.DeviceID)
+					deviceID = p.Spec.NicSelector.DeviceID
 				} else {
-					rc.Selectors.Devices = append(rc.Selectors.Devices, sriovnetworkv1.SriovPfVfMap[p.Spec.NicSelector.DeviceID])
+					deviceID = sriovnetworkv1.GetVfDeviceId(p.Spec.NicSelector.DeviceID)
+				}
+
+				if !sriovnetworkv1.StringInArray(deviceID, netDeviceSelectors.Devices) && deviceID != "" {
+					netDeviceSelectors.Devices = append(netDeviceSelectors.Devices, deviceID)
 				}
 			}
 			if l := len(p.Spec.NicSelector.PfNames); l > 0 {
-				rc.Selectors.PfNames = append(rc.Selectors.PfNames, p.Spec.NicSelector.PfNames...)
+				netDeviceSelectors.PfNames = append(netDeviceSelectors.PfNames, p.Spec.NicSelector.PfNames...)
+			}
+			// Removed driver constraint for "netdevice" DeviceType
+			if p.Spec.DeviceType == "vfio-pci" {
+				netDeviceSelectors.Drivers = append(netDeviceSelectors.Drivers, p.Spec.DeviceType)
 			}
 
-			if p.Spec.DeviceType == "vfio-pci" {
-				rc.Selectors.Drivers = append(rc.Selectors.Drivers, p.Spec.DeviceType)
-			} else {
-				if p.Spec.NumVfs > 0 {
-					///////////////////////////////////
-					// TODO: remove unsupport VF driver
-					///////////////////////////////////
-					rc.Selectors.Drivers = append(rc.Selectors.Drivers, "iavf", "mlx5_core", "i40evf", "ixgbevf")
-				}
+			netDeviceSelectorsMarshal, err := json.Marshal(netDeviceSelectors)
+			if err != nil {
+				return rcl, err
 			}
+			rawNetDeviceSelectors := json.RawMessage(netDeviceSelectorsMarshal)
+			rc.Selectors = &rawNetDeviceSelectors
 			rcl.ResourceList = append(rcl.ResourceList, *rc)
 			logger.Info("Add resource", "Resource", *rc, "Resource list", rcl.ResourceList)
 		}
